# -*- coding: utf-8 -*-
"""Define project metadata
"""

__package_name__ = 'korona'
__description__ = 'Package for building html pages'
__url__ = 'https://github.com/bharadwajyarlagadda/korona'
__version__ = '0.3.1'
__author__ = 'Bharadwaj Yarlagadda'
__email__ = 'yarlagaddabharadwaj@gmail.com'
__license__ = 'MIT License'
